/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LADCCA_COMM_EVENT_H__
#define __LADCCA_COMM_EVENT_H__

#include <ladcca/internal_headers.h>

enum CCA_Comm_Event_Type
{
  CCA_Comm_Event_Connect = 1,
  CCA_Comm_Event_Event,
  CCA_Comm_Event_Config,
  CCA_Comm_Event_Ping,
  CCA_Comm_Event_Pong,
  CCA_Comm_Event_Close,
  CCA_Comm_Event_Protocol_Mismatch
};

struct _cca_comm_event   
{
  enum CCA_Comm_Event_Type type;
  union
  {
    cca_client_t * client;
    cca_event_t * event;  
    cca_config_t * config;
    char * string;
    long number;
  } event_data;
};


void cca_comm_event_init (cca_comm_event_t * event);
void cca_comm_event_free (cca_comm_event_t * event);

cca_comm_event_t * cca_comm_event_new ();
void               cca_comm_event_destroy (cca_comm_event_t * event);

void cca_comm_event_set_type              (cca_comm_event_t *, enum CCA_Comm_Event_Type);
void cca_comm_event_set_connect           (cca_comm_event_t *, cca_client_t *);
void cca_comm_event_set_event             (cca_comm_event_t *, cca_event_t *);
void cca_comm_event_set_config            (cca_comm_event_t *, cca_config_t *);
void cca_comm_event_set_protocol_mismatch (cca_comm_event_t *, cca_protocol_t);

enum CCA_Comm_Event_Type  cca_comm_event_get_type (cca_comm_event_t *);

cca_event_t *   cca_comm_event_take_event  (cca_comm_event_t * event);
cca_config_t *  cca_comm_event_take_config (cca_comm_event_t * event);
cca_client_t *  cca_comm_event_take_client (cca_comm_event_t * event);
long            cca_comm_event_get_number  (const cca_comm_event_t * event);

#endif /* __LADCCA_COMM_EVENT_H__ */
