/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LADCCAD_ALSA_PATCH_H__
#define __LADCCAD_ALSA_PATCH_H__

#include <uuid/uuid.h>
#include <alsa/asoundlib.h>
#include <ladcca/ladcca.h>

typedef struct _alsa_patch alsa_patch_t;

struct _alsa_patch
{
  uuid_t from_id;
  uuid_t to_id;
  snd_seq_port_subscribe_t * sub;
};

void alsa_patch_init (alsa_patch_t * patch);
void alsa_patch_free (alsa_patch_t * patch);

alsa_patch_t * alsa_patch_new            ();
alsa_patch_t * alsa_patch_dup            (const alsa_patch_t * patch);
alsa_patch_t * alsa_patch_new_with_sub   (const snd_seq_port_subscribe_t * sub);
alsa_patch_t * alsa_patch_new_with_query (const snd_seq_query_subscribe_t * query);
void           alsa_patch_destroy        (alsa_patch_t * patch);

void                             alsa_patch_get_from_id        (const alsa_patch_t * patch, uuid_t id);
void                             alsa_patch_get_to_id          (const alsa_patch_t * patch, uuid_t id);
const snd_seq_port_subscribe_t * alsa_patch_get_sub            (const alsa_patch_t * patch);
unsigned char                    alsa_patch_get_from_client    (const alsa_patch_t * patch);
unsigned char                    alsa_patch_get_from_port      (const alsa_patch_t * patch);
unsigned char                    alsa_patch_get_to_client      (const alsa_patch_t * patch);
unsigned char                    alsa_patch_get_to_port        (const alsa_patch_t * patch);

int  alsa_patch_from_id_is_null (alsa_patch_t * patch);
int  alsa_patch_to_id_is_null   (alsa_patch_t * patch);

void alsa_patch_set_from_id        (alsa_patch_t * patch, uuid_t id);
void alsa_patch_set_to_id          (alsa_patch_t * patch, uuid_t id);
void alsa_patch_set_sub            (alsa_patch_t * patch, const snd_seq_port_subscribe_t * sub);
void alsa_patch_set_sub_from_query (alsa_patch_t * patch, const snd_seq_query_subscribe_t * query);

const char * alsa_patch_get_spec (alsa_patch_t * patch);
void         alsa_patch_set_spec (alsa_patch_t * patch, const char * spec);

void alsa_patch_set   (alsa_patch_t * patch, cca_list_t * clients);
int  alsa_patch_unset (alsa_patch_t * patch, cca_list_t * clients);

const snd_seq_port_subscribe_t * query_to_port (const snd_seq_query_subscribe_t * query);
void alsa_patch_switch_clients (alsa_patch_t * patch);

#endif /* __LADCCAD_ALSA_PATCH_H__ */
