/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <ladcca/internal_headers.h>

#include "client.h"


void
client_init (client_t * client)
{
  client->cca_client = NULL;
  client->name = NULL;
  client->store = NULL;
  client->jack_client_name = NULL;
  client->jack_patches = NULL;
  client->alsa_client_id = 0;
  client->alsa_patches = NULL;
}

void
client_free (client_t * client)
{
  cca_client_destroy (client->cca_client);
  client_set_name (client, NULL);
  client_set_jack_client_name (client, NULL);
  if (client->store)
    store_destroy (client->store);
}

client_t *
client_new ()
{
  client_t * client;
  client = cca_malloc (sizeof (client_t));
  client_init (client);
  return client;
}

void
client_destroy (client_t * client)
{
  client_free (client);
  free (client);
}

cca_client_t *
client_get_cca_client (client_t * client)
{
  return client->cca_client;
}


const char *
client_get_name (client_t * client)
{
  return client->name;
}

unsigned long
client_get_conn_id (client_t * client)
{
  return client->conn_id;
}

store_t *
client_get_store (client_t * client)
{
  return client->store;
}

const char *
client_get_jack_client_name (client_t * client)
{
  return client->jack_client_name;
}

unsigned char
client_get_alsa_client_id   (client_t * client)
{
  return client->alsa_client_id;
}


void client_set_cca_client (client_t * client, cca_client_t * cca_client)
{
  if (client->cca_client)
    cca_client_destroy (client->cca_client);
  
  client->cca_client = cca_client;
}


void
client_set_name (client_t * client, const char * name)
{
  if (client->name)
    free (client->name);
  
  if (name)
    client->name = cca_strdup (name);
  else
    client->name = NULL;
}

void
client_set_conn_id (client_t * client, unsigned long id)
{
  client->conn_id = id;
}

void
client_generate_id (client_t * client)
{
  uuid_t id;
  uuid_generate (id);
  cca_client_set_id (client_get_cca_client (client), id);
}

int
client_store_open (client_t * client, const char * dir)
{
  int err;

  if (client->store)
    store_destroy (client->store);
  
  err = store_open (dir, &client->store);
  if (err)
    fprintf (stderr, "%s: WARNING: the store for client '%s' (in directory '%s') could not be read. You should resolve this before saving the project.  You may also wish to restart the client after it has been resolved, as it will not be given any data from the store.\n",
             __FUNCTION__, cca_client_get_id_str (client->cca_client), store_get_dir (client->store));
  
  return err;
}

cca_config_t *
client_store_get_config (client_t * client, const char * key)
{
  return store_get_config (client->store, key);
}

int
client_store_write      (client_t * client)
{
  return store_write (client->store);
}


void
client_set_jack_client_name (client_t * client, const char * name)
{
  if (client->jack_client_name)
    free (client->jack_client_name);
  
  if (name)
    client->jack_client_name = cca_strdup (name);
  else
    client->jack_client_name = NULL;
}

void
client_set_alsa_client_id   (client_t * client, unsigned char id)
{
  client->alsa_client_id = id;
}


/* EOF */

