/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LADCCAD_CLIENT_H__
#define __LADCCAD_CLIENT_H__

#include <uuid/uuid.h>

#include <ladcca/ladcca.h>

#include "store.h"

typedef struct _client client_t;

struct _client
{
  cca_client_t *        cca_client;
  unsigned long         conn_id;
  char *                name;
  store_t *             store;
  
  char *                jack_client_name;
  cca_list_t *          jack_patches;
  unsigned char         alsa_client_id;
  cca_list_t *          alsa_patches;
};

void client_init (client_t * client);
void client_free (client_t * client);

client_t * client_new ();
void       client_destroy (client_t * client);

cca_client_t *  client_get_cca_client       (client_t * client);
const char *    client_get_name             (client_t * client);
unsigned long   client_get_conn_id          (client_t * client);
store_t *       client_get_store            (client_t * client);
const char *    client_get_jack_client_name (client_t * client);
unsigned char   client_get_alsa_client_id   (client_t * client);

void client_set_cca_client       (client_t * client, cca_client_t * cca_client);
void client_set_name             (client_t * client, const char * name);
void client_set_conn_id          (client_t * client, unsigned long id);
void client_set_jack_client_name (client_t * client, const char * name);
void client_set_alsa_client_id   (client_t * client, unsigned char id);

void client_generate_id          (client_t * client);
/*void client_restore_jack_patches (client_t * client);
void client_restore_alsa_patches (client_t * client); */

int             client_store_open       (client_t * client, const char * dir);
cca_config_t *  client_store_get_config (client_t * client, const char * key);
int             client_store_write      (client_t * client);

#endif /* __LADCCAD_CLIENT_H__ */
