/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE

#include <unistd.h>
#include <errno.h>

#include <ladcca/ladcca.h>
#include <ladcca/internal_headers.h>

#include "client_event.h"
#include "loader.h"

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN  64
#endif


void
server_cca_event_client_name (project_t * project, client_t * client, const char * name)
{
  project_name_client (project, client, name);
}

void
server_cca_event_jack_client_name (server_t * server, client_t * client, const char * name)
{
  if (name)
    {
      cca_client_t * cca_client;
      cca_client = client_get_cca_client (client);
      
      CCA_DEBUGARGS ("setting jack client name for client '%s' to '%s'",
                     cca_client_get_id_str (cca_client), name);
      
      if (client_get_jack_client_name (client))
        {
          fprintf (stderr, "%s client '%s' sent more than one jack client name\n",
                   __FUNCTION__, cca_client_get_id_str (cca_client));
          return;
        }
                     
      client_set_jack_client_name (client, name);

      if (!CCA_CLIENT_USES_JACK (cca_client))
        cca_client_set_flags (cca_client, CCA_Use_Jack | cca_client_get_flags (cca_client));
      
      jack_mgr_add_client (server->jack_mgr, cca_client->id, name, client->jack_patches);
      client->jack_patches = NULL; 
    }
  else
    {
      cca_event_t * event;
      event = cca_event_new_with_type (CCA_Jack_Client_Name);
      cca_event_set_string (event, client_get_jack_client_name (client));
      conn_mgr_send_client_cca_event (server_get_conn_mgr (server),
                                      client_get_conn_id (client),
                                      event);
    }
}

void
server_cca_event_alsa_client_id   (server_t * server, project_t * project, client_t * client, const char * string)
{
  if (string)
    {
      unsigned char id;
      cca_client_t * cca_client;
      
      cca_client = client_get_cca_client (client);

      CCA_DEBUGARGS ("setting alsa client id for client '%s' to %d",
                     cca_client_get_id_str (cca_client), (unsigned char) string[0]);
                     
      if (client_get_alsa_client_id (client))
        {
          fprintf (stderr, "%s: client '%s' sent more than one alsa client id\n",
                   __FUNCTION__, cca_client_get_id_str (cca_client));
          return;
        }

      id = (unsigned char) string[0];
      client_set_alsa_client_id (client, id);

      if (!CCA_CLIENT_USES_ALSA (cca_client))
        cca_client_set_flags (cca_client, CCA_Use_Alsa | cca_client_get_flags (cca_client));
      
      alsa_mgr_add_client (server->alsa_mgr, cca_client->id, id, client->alsa_patches);
      client->alsa_patches = NULL;
    }
  else
    {
      cca_event_t * event;
      char id[2];


      event = cca_event_new_with_type (CCA_Alsa_Client_ID);

      if (client_get_alsa_client_id (client))
        {
          id[0] = client_get_alsa_client_id (client);
          id[1] = '\0';
          cca_event_set_string (event, id);
        }

      conn_mgr_send_client_cca_event (server_get_conn_mgr (server),
                                      client_get_conn_id (client),
                                      event);
    }
}

void
server_cca_event_restore_data_set (server_t * server, project_t * project, client_t * client)
{
/*  project_restore_data_set (project, client); */
}

void
server_cca_event_save (server_t * server, project_t * project, client_t * client)
{
  if (strcmp (project_get_name (project), UI_PROJECT_NAME) == 0)
    {
      cca_client_t * cca_client;
      cca_client = client_get_cca_client (client);
      project = server_find_project_by_name (server, cca_client_get_project_name (cca_client));
      if (!project)
        {
          fprintf (stderr, "%s: ui client '%s' is connected to unknown project '%s'; can't save\n", 
                   __FUNCTION__, cca_client_get_id_str (cca_client), cca_client_get_project_name (cca_client));
          return;
        }
    }
  
  project_save (project);
}

void
server_cca_event_project		(server_t * server, project_t * project, client_t * client, const char * name)
{
  project_t * new_project;
  cca_client_t * cca_client;
  
  cca_client = client_get_cca_client (client);
  
  CCA_DEBUGARGS ("changing project for client '%s' from '%s' to '%s'",
                 cca_client_get_id_str (cca_client),
                 project_get_name (project), name);
  
  
  /* project request */
  if (!name)
    {
      cca_event_t * event;
      event = cca_event_new_with_type (CCA_Project);
      cca_event_set_string (event, project_get_name (project));
      conn_mgr_send_client_cca_event (server_get_conn_mgr (server), client_get_conn_id (client), event);
      return;
    }
  
  new_project = server_find_project_by_name (server, name);
  if (!new_project)
    {
      fprintf (stderr, "%s: could not change project for client '%s' to '%s': no such project\n",
               __FUNCTION__, cca_client_get_id_str (cca_client), name);
      return;
    }
  
  
  /* ui clients */
  if (strcmp (project_get_name (project), UI_PROJECT_NAME) == 0)
    {
      cca_client_set_project_name (cca_client, name);
      return;
    }
  
  if (project == new_project)
    return;
    
  if (CCA_CLIENT_CONFIGURED_WITH_DATA_SET (cca_client) ||
      CCA_CLIENT_CONFIGURED_WITH_FILE     (cca_client))
    {
/*      char * old_dir;
      const char * new_dir;
      
      old_dir = cca_strdup (project_get_client_id_dir (project, client));
      new_dir = project_get_client_id_dir (new_project, client);
      
      err = rename (old_dir, new_dir);
      if (err == -1)
        {
          fprintf (stderr, "%s: could not move id dir '%s' to new dir '%s': %s\n",
                   __FUNCTION__, old_dir, new_dir, strerror (errno));
          free (old_dir);
          return;
        }
      free (old_dir);
      
      store_set_dir (client_get_store (client), new_dir);
      
      if (client_get_name (client))
        {
          old_dir = (char *) project_get_client_name_dir (project, client);
          err = unlink (old_dir);
          if (err == -1)
            {
              fprintf (stderr, "%s: could not remove old name link '%s': %s\n",
                       __FUNCTION__, old_dir, strerror (errno));
            }
          
          old_dir = cca_strdup (project_get_client_id_dir (new_project, client));
          new_dir = project_get_client_name_dir (new_project, client);
          err = symlink (old_dir, new_dir);
          if (err == -1)
            {
              fprintf (stderr, "%s: could not create name link '%s' for client '%s': %s\n",
                       __FUNCTION__, new_dir, cca_client_get_id_str (cca_client), strerror (errno));
            }
          free (old_dir);
        }*/
      
      project_move_client (project, client, project_get_directory (new_project));
    }

  project->clients = cca_list_remove (project->clients, client);
  new_project->clients = cca_list_append (new_project->clients, client);
  
}

void
server_cca_event_project_dir      (server_t * server, project_t * project, client_t * client, const char * dir)
{
}

void
server_cca_event_project_name     (server_t * server, project_t * project, client_t * client, const char * name)
{
}

void
server_cca_event_projects         (server_t * server, client_t * client)
{
  cca_list_t * list;
  cca_event_t * event;
  project_t * project;
  char * string, * ptr;
  size_t string_size = 0;
  
  event = cca_event_new_with_type (CCA_Projects);
  
  if (cca_list_length (server->projects) == 1)
    {
      cca_event_set_string (event, "");
      conn_mgr_send_client_cca_event (server_get_conn_mgr (server),
                                      client_get_conn_id (client),
                                      event);
      return;
    }
  
  list = server->projects;
  while (list)
    {
      project = (project_t *) list->data;
      
      if (strcmp (project_get_name (project), UI_PROJECT_NAME) != 0)
        string_size += strlen (project_get_name (project)) + 1;
      
      list = list->next;
    }
  
  string = cca_malloc (string_size);
  ptr = string;
  
  list = server->projects;
  while (list)
    {
      project = (project_t *) list->data;
      
      if (strcmp (project_get_name (project), UI_PROJECT_NAME) != 0)
        {
          if (ptr != string)
            {
              *ptr = ':';
              ptr++;
            }

          string_size = strlen (project_get_name (project));
          memcpy (ptr, project_get_name (project), string_size);
          ptr += string_size;
      
        }
     
      list = list->next;
    }
  
  *ptr = '\0';
  
  CCA_DEBUGARGS ("sending project list '%s' to client '%s'", string,
                 cca_client_get_id_str (client_get_cca_client (client)));
  cca_event_set_string (event, string);

  free (string);
  
  conn_mgr_send_client_cca_event (server_get_conn_mgr (server), client_get_conn_id (client), event);
                                  
}

void
server_cca_event_restore          (server_t * server, const char * dir)
{
  project_t * project;
  client_t * client;
  cca_list_t * list;
  cca_client_t * cca_client;
  char server_name[MAXHOSTNAMELEN];
  int err;
  
  project = project_restore (server, dir);
  if (!project)
    {
      fprintf (stderr, "%s: could not restore project in dir '%s'\n",
               __FUNCTION__, dir);
      return;
    }
  
  if (project_name_exists (server->projects, project_get_name (project)))
    {
      const char * name;
      name = server_create_new_project_name (server);
      fprintf (stderr, "%s: changing project name for project in dir '%s' to '%s' to avoid clashing with existing project name '%s'\n",
               __FUNCTION__, dir, name, project_get_name (project));
      project_set_name (project, name);
    }
  
/*  project_set_conn_mgr (project, server_get_conn_mgr (server)); */
  
  err = gethostname (server_name, MAXHOSTNAMELEN);
  if (err == -1)
    {
      fprintf (stderr, "%s: could not get local host name, using 'localhost' instead: %s\n",
               __FUNCTION__, strerror (errno));
      strcpy (server_name, "localhost");
    }
    
  server->projects = cca_list_append (server->projects, project);
  
  list = project->lost_clients;
  while (list)
    {
      client = (client_t *) list->data;
      cca_client = client_get_cca_client (client);
      
      cca_client_set_server_name (cca_client, server_name);
      cca_client_set_project_name (cca_client, project_get_name (project));
      cca_client->server_connected = server->conn_mgr->port;
      
      loader_load (server->loader, cca_client);
      
      list = list->next;
    }
  
}

/* tell the store to write to disk */
void
server_cca_event_save_data_set    (client_t * client)
{
  int err;
  
  if (!CCA_CLIENT_CONFIGURED_WITH_DATA_SET (client->cca_client))
    return;
  
  err = client_store_write (client);
  if (err)
    fprintf (stderr, "%s: could not write client '%s's data to disk!  You should attempt to ascertain why, resolve the situation, and save the project again.\n",
             __FUNCTION__, cca_client_get_id_str (client->cca_client));
}

/* EOF */

