/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LADCCAD_JACK_PATCH_H__
#define __LADCCAD_JACK_PATCH_H__

#include <uuid/uuid.h>

#include "project.h"

typedef struct _jack_patch jack_patch_t;

struct _jack_patch
{
  char * from;
  char * to;
  uuid_t from_id;
  uuid_t to_id;
};

void jack_patch_init (jack_patch_t * patch);
void jack_patch_free (jack_patch_t * patch);

jack_patch_t * jack_patch_new ();
jack_patch_t * jack_patch_dup (const jack_patch_t * patch);
void           jack_patch_destroy (jack_patch_t * patch);

const char * jack_patch_get_from      (const jack_patch_t * patch);
const char * jack_patch_get_to        (const jack_patch_t * patch);
const char * jack_patch_get_from_port (const jack_patch_t * patch);
const char * jack_patch_get_to_port   (const jack_patch_t * patch);
const char * jack_patch_get_from_client (const jack_patch_t * patch);
const char * jack_patch_get_to_client   (const jack_patch_t * patch);
void         jack_patch_set_from      (jack_patch_t * patch, const char * from);
void         jack_patch_set_to        (jack_patch_t * patch, const char * to);
void         jack_patch_get_from_id   (const jack_patch_t * patch, uuid_t id);
void         jack_patch_set_from_id   (jack_patch_t * patch, uuid_t id);
void         jack_patch_get_to_id     (const jack_patch_t * patch, uuid_t id);
void         jack_patch_set_to_id     (jack_patch_t * patch, uuid_t id);

/* set the IDs and strings to the IDs of project clients */
void         jack_patch_set         (jack_patch_t * patch, cca_list_t * jack_mgr_clients);
int          jack_patch_unset       (jack_patch_t * patch, cca_list_t * jack_mgr_clients);

void         jack_patch_set_spec    (jack_patch_t * patch, const char * spec);
const char * jack_patch_get_spec    (jack_patch_t * patch);

void jack_patch_switch_clients (jack_patch_t * patch);

#endif /* __LADCCAD_JACK_PATCH_H__ */
