/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <mcheck.h>

#include <jack/jack.h>

#include <ladcca/ladcca.h>
#include <ladcca/debug.h>

#include "ac-config.h"
#include "conn_mgr.h"
#include "server.h"
#include "loader.h"

server_t *   global_server   = NULL;

void
term_handler (int signum)
{
  CCA_DEBUGARGS ("recieved signal %d (%s), terminating", signum, strsignal (signum));

  global_server->quit = 1;
}

/*void
loader_quit_handler (int signum)
{
  CCA_DEBUGARGS ("recieved signal %s; setting loader's quit flag", strsignal (signum));
  
  server_set_loader_quit (global_server, 1);
}*/

static void
print_help ()
{
  printf("ladccad version %s\n", PACKAGE_VERSION);
  printf("Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)\n");
  printf("\n");
  printf("This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it\n");
  printf("under the terms of the GNU General Public License, version 2 or later.  See\n");
  printf("the COPYING file that came with this software for details.\n");
  printf("\n");
  printf("Compiled with ALSA %s and JACK %s\n", ALSA_VERSION, JACK_VERSION);
  printf("\n");
  printf(" -h, --help                  Display this help info\n");
#ifdef HAVE_JACK_SET_SERVER_DIR
  printf(" -D, --tmpdir <dir>          Tell JACK to use <dir> for its temporary files\n");
#endif
  printf(" -p, --port <port>           Listen on port <port>\n");
  printf(" -d, --default-dir <dir>     Use <dir> to store project directories\n");
  printf("\n");
}
                            

int
main (int argc, char ** argv)
{
  int opt;
#ifdef HAVE_JACK_SET_SERVER_DIR
  const char * options = "hD:p:d:";
#else
  const char * options = "hp:d:";
#endif
  struct option long_options[] = {
    { "help", 0, NULL, 'h' },
#ifdef HAVE_JACK_SET_SERVER_DIR
    { "tmpdir", 1, NULL, 'D' },
#endif
    { "port", 1, NULL, 'p' },
    { "default-dir", 1, NULL, 'd' },
    { 0, 0, 0, 0 }
  };
  char * default_dir = NULL;
  int port = 0;
  sighandler_t sigh;

  
#ifdef LADCCA_DEBUG
  mtrace ();
#endif 




  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1)
    {
      switch (opt)
      {
      case 'h':
        print_help ();
        exit (0);
        break;
#ifdef HAVE_JACK_SET_SERVER_DIR
      case 'D':
        jack_set_server_dir (optarg);
        break;
#endif
      case 'd':
        default_dir = cca_strdup (optarg);
        break;
        
      case 'p':
      {
        long l;
        char * ptr;
        
        l = strtol (optarg, &ptr, 10);
        
        if (l == LONG_MAX ||
            l == LONG_MIN ||
            l <= 0 ||
            l > UINT_MAX ||
            ptr == optarg)
          port = CCA_DEFAULT_PORT;
        else
          port = l;
        
        break;
      }
      
      case ':':
        print_help ();
        exit (1);
        break;
      case '?':
        print_help ();
        exit (1);
        break;
    }
  }

  if (!default_dir)
    default_dir = cca_strdup (DEFAULT_PROJECT_DIR);
  
  if (!port)
    port = CCA_DEFAULT_PORT;
  


  global_server = server_new (port, default_dir);

  /* install the signal handlers */
  sigh = signal (SIGTERM, term_handler);
  if (sigh == SIG_IGN)
    signal (SIGTERM, SIG_IGN);
    
  sigh = signal (SIGINT, term_handler);
  if (sigh == SIG_IGN)
    signal (SIGINT, SIG_IGN);
    
  sigh = signal (SIGHUP, term_handler);
  if (sigh == SIG_IGN)
    signal (SIGHUP, SIG_IGN);

  signal (SIGCHLD, term_handler);
  signal (SIGPIPE, SIG_IGN);



  server_main (global_server);
  
  printf ("Cleaning up\n");
    
  CCA_DEBUG ("finished, cleaning up");
  
  server_destroy (global_server);
  global_server = NULL;
  
  free (default_dir);
  
  CCA_DEBUG ("cleaned up, exiting");
  printf ("Finished\n");
  exit (0);
}

