/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <ladcca/types.h>
#include <ladcca/client.h>
#include <ladcca/xmalloc.h>
#include <ladcca/internal.h>

void
cca_client_init (cca_client_t * client)
{
  client->project_name = NULL;
  client->flags = 0;
  client->argc = 0;
  client->argv = NULL;
  client->working_dir = NULL;
  client->comm = NULL;
  client->server_name = NULL;
  client->server_connected = 0;
  client->class = NULL;
  uuid_clear (client->id);
}

void
cca_client_free (cca_client_t * client)
{
  cca_client_set_project_name (client, NULL);
  cca_client_set_working_dir  (client, NULL);
  cca_client_set_server_name  (client, NULL);
  cca_client_set_class        (client, NULL);

  client->flags = 0;
  client->argc = 0;
  client->comm = NULL;
  uuid_clear (client->id);
}

cca_client_t *
cca_client_new ()
{
  cca_client_t * client;
  client = cca_malloc (sizeof (cca_client_t));
  cca_client_init (client);
  return client;
}

void
cca_client_destroy (cca_client_t * client)
{
  cca_client_free (client);
  free (client);
}

cca_client_t *
cca_client_dup     (const cca_client_t * other)
{
  cca_client_t * client;
  
  client = cca_client_new ();
  
  cca_client_set_project_name     (client, cca_client_get_project_name (other));
  cca_client_set_working_dir      (client, cca_client_get_working_dir (other));
  cca_client_set_server_name      (client, cca_client_get_server_name (other));
  cca_client_set_flags            (client, cca_client_get_flags (other));
  cca_client_set_args             (client, cca_client_get_argc (other), cca_client_get_argv (other));
  cca_client_set_id               (client, ((cca_client_t *) other)->id);
  cca_client_set_class            (client, cca_client_get_class (other));
  cca_client_set_server_connected (client, cca_client_get_server_connected (other));
  
  return client;
}

const char *
cca_client_get_project_name (const cca_client_t * client)
{
  return client->project_name;
}

enum CCA_Client_Flags
cca_client_get_flags (const cca_client_t * client)
{
  return client->flags;
}


const char *
cca_client_get_working_dir (const cca_client_t * client)
{
  return client->working_dir;
}

int
cca_client_get_argc (const cca_client_t * client)
{
  return client->argc;
}

char **
cca_client_get_argv (const cca_client_t * client)
{
  return client->argv;
}

struct _cca_comm *
cca_client_get_comm (const cca_client_t * client)
{
  return client->comm;
}

const char *
cca_client_get_server_name (const cca_client_t * client)
{
  return client->server_name;
}

const char *
cca_client_get_class (const cca_client_t * client)
{
  return client->class;
}

void
cca_client_get_id (const cca_client_t * client, uuid_t id_copy)
{
  uuid_copy (id_copy, client->id);
}

const char *
cca_client_get_id_str (const cca_client_t * client)
{
  static char id[37];
  uuid_unparse (client->id, id);
  return id;
}

void
cca_client_set_project_name (cca_client_t * client, const char * project)
{
  set_string_property (client->project_name, project)
}

void
cca_client_set_flags (cca_client_t * client, enum CCA_Client_Flags flags)
{
  client->flags = flags;
}


void
cca_client_set_working_dir (cca_client_t * client, const char * working_dir)
{
/*  if (client->working_dir)
    free (client->working_dir);
  
  if (working_dir)
    client->working_dir = cca_strdup (working_dir);
  else
    client->working_dir = NULL;*/
  set_string_property (client->working_dir, working_dir);
}

void
cca_client_set_server_name (cca_client_t * client, const char * server_name)
{
/*  if (client->server_name)
    free (client->server_name);
  
  if (server_name)
    client->server_name = cca_strdup (server_name);
  else
    client->server_name = NULL;*/
  set_string_property (client->server_name, server_name);
}


void
cca_client_set_class (cca_client_t * client, const char * class)
{
  set_string_property (client->class, class);
}

void
cca_client_set_args (cca_client_t * client, int argc, char ** argv)
{
  client->argc = argc;
  client->argv = argv;
}

void
cca_client_set_comm (cca_client_t * client, struct _cca_comm * comm)
{
  client->comm = comm;
}

void
cca_client_set_id (cca_client_t * client, uuid_t id)
{
  uuid_copy (client->id, id);
}

uint32_t
cca_client_get_server_connected (const cca_client_t * client)
{
  return client->server_connected;
}

void
cca_client_set_server_connected (cca_client_t * client, uint32_t connected)
{
  client->server_connected = connected;
}


/* EOF */

