/*
 *   LADCCA
 *    
 *   Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <netinet/in.h>

#include <ladcca/ladcca.h>
#include <ladcca/internal_headers.h>

void 
cca_buffer_from_comm_event_connect (char ** buf_ptr, size_t * buf_size_ptr, cca_client_t * client)
{
  size_t buf_size;
  char * buf;
  size_t proj_name_size;
  size_t working_dir_size;
  size_t server_name_size;
  size_t class_size;
  size_t args_size;
  int i;
  uint32_t * iptr;
  char * ptr;
  uuid_t id;
  char id_str[37];
  
  
  buf_size = sizeof (uint32_t) * 4;
  buf_size += proj_name_size = strlen (cca_client_get_project_name (client)) + 1;
  buf_size += working_dir_size = strlen (cca_client_get_working_dir (client)) + 1;
  buf_size += class_size = strlen (cca_client_get_class (client)) + 1;
  if (cca_client_get_server_name (client))
    buf_size += server_name_size = strlen (cca_client_get_server_name (client)) + 1;
  else
    buf_size += server_name_size = 1;
  buf_size += sizeof (id_str);
  
  args_size = 0;
  for (i = 0; i < client->argc; i++)
    args_size += strlen (client->argv[i]) + 1;
  buf_size += args_size;
  
  buf = cca_malloc (buf_size);

  
  iptr = (uint32_t *) buf;
  *iptr = htonl (CCA_Comm_Event_Connect);
  iptr++;

  *iptr = htonl (client->flags);
  iptr++;
  
  /* this is for the loader, which uses it for the port number */
  *iptr = htonl (client->server_connected);
  iptr++;
  
  memcpy (iptr, cca_client_get_project_name (client), proj_name_size);
  ptr = (char *) iptr;
  ptr += proj_name_size;
  
  memcpy (ptr, cca_client_get_working_dir (client), working_dir_size);
  ptr += working_dir_size;

  memcpy (ptr, cca_client_get_class (client), class_size);
  ptr += class_size;
  
  if (cca_client_get_server_name (client))
    memcpy (ptr, cca_client_get_server_name (client), server_name_size);
  else
    memcpy (ptr, "", server_name_size);
  ptr += server_name_size;
  
  cca_client_get_id (client, id);
  uuid_unparse (id, id_str);
  memcpy (ptr, id_str, sizeof (id_str));
  ptr += sizeof (id_str);
    
  iptr = (uint32_t *) ptr;
  *iptr = htonl (client->argc);
  iptr++;
  
  ptr = (char *) iptr;
  for (i = 0; i < client->argc; i++)
    {
      args_size = strlen (client->argv[i]) + 1;
      memcpy (ptr, client->argv[i], args_size);
      ptr += args_size;
    }

  *buf_ptr = buf;
  *buf_size_ptr = buf_size;    
}

void
cca_comm_event_from_buffer_connect (char * buf, size_t buf_size, cca_comm_event_t * event)
{
  uint32_t * iptr;
  char * ptr;
  cca_client_t * client;
  size_t argv_size;
  int i;
  uuid_t id;
  
  iptr = (uint32_t *) buf;
  event->type = ntohl (*iptr);
  iptr++;
  
  client = cca_client_new ();
  
  cca_client_set_flags (client, ntohl (*iptr));
  iptr++;
  
  cca_client_set_server_connected (client, ntohl (*iptr));
  iptr++;
  
  ptr = (char *) iptr;
  cca_client_set_project_name (client, ptr);
  ptr += strlen (ptr) + 1;

  cca_client_set_working_dir (client, ptr);
  ptr += strlen (ptr) + 1;
  
  cca_client_set_class (client, ptr);
  ptr += strlen (ptr) + 1;

  cca_client_set_server_name (client, ptr);
  ptr += strlen (ptr) + 1;
  
  uuid_parse (ptr, id);
  cca_client_set_id (client, id);
  ptr += sizeof (char[37]);
  
  iptr = (uint32_t *) ptr;
  client->argc = ntohl (*iptr);
  iptr++;
  
  client->argv = cca_malloc (client->argc * sizeof (char *));
  ptr = (char *) iptr;
  
  /* get the argv size */
  argv_size = 0;
  for (i = 0; i < client->argc; i++)
    {
      argv_size += strlen (ptr) + 1;
      ptr += strlen (ptr) + 1;
    }
  
  /* create it */
  client->argv[0] = cca_malloc (argv_size);
  memcpy (client->argv[0], iptr, argv_size);
  
  /* set the argv array */
  ptr = client->argv[0];
  for (i = 0; i < client->argc; i++)
    {
      client->argv[i] = ptr;
      ptr = strchr (ptr, '\0');
      ptr++;
    }
  
  event->event_data.client = client;
}




void
cca_buffer_from_comm_event_event (char ** buf_ptr, size_t * buf_size_ptr, cca_event_t * event)
{
  size_t buf_size = sizeof (uint32_t) * 2;
  char * buf;
  size_t string_size = 0;
  uint32_t * iptr;
  
  if (event->string)
    {
      buf_size += string_size = strlen (event->string) + 1;
    }
  
  buf = cca_malloc (buf_size);
  
  
  iptr = (uint32_t *) buf;
  *iptr = htonl (CCA_Comm_Event_Event);
  iptr++;
  
  *iptr = htonl (event->type);
  iptr++;
  
  if (event->string)
    memcpy (iptr, event->string, string_size);
    
  *buf_ptr = buf;
  *buf_size_ptr = buf_size;
}

void
cca_comm_event_from_buffer_event (char * buf, size_t buf_size, cca_comm_event_t * comm_event)
{
  uint32_t * iptr;
  char * ptr;
  cca_event_t * event;
  
  comm_event->type = CCA_Comm_Event_Event;
  iptr = (uint32_t *) buf;
  iptr++;
  
  event = cca_event_new ();
  
  cca_event_set_type (event, ntohl (*iptr));
  iptr++;
  
  if (buf_size > sizeof (uint32_t) * 2)
    {
      ptr = (char *) iptr;
      cca_event_set_string (event, ptr);
    }
  
  comm_event->event_data.event = event;
}






void
cca_buffer_from_comm_event_config (char ** buf_ptr, size_t * buf_size_ptr, cca_config_t * config)
{
  size_t buf_size = sizeof (uint32_t);
  char * buf;
  size_t key_size;
  uint32_t * iptr;
  char * ptr;
  
  buf_size += key_size = strlen (config->key) + 1;
  if (config->value)
    {
      buf_size += sizeof (uint32_t);
      buf_size += config->value_size;
    }
  
  buf = cca_malloc (buf_size);
  
  
  iptr = (uint32_t *) buf;
  *iptr = htonl (CCA_Comm_Event_Config);
  iptr++;
  
  memcpy (iptr, config->key, key_size);
  ptr = (char *) iptr;
  ptr += key_size;
  
  if (config->value)
    {
      iptr = (uint32_t *) ptr;
      *iptr = htonl (config->value_size);
      iptr++;
      
      memcpy (iptr, config->value, config->value_size);
    }

  *buf_ptr = buf;
  *buf_size_ptr = buf_size;
}

void
cca_comm_event_from_buffer_config (char * buf, size_t buf_size, cca_comm_event_t * event)
{
  uint32_t * iptr;
  char * ptr;
  cca_config_t * config;
  size_t key_size;
  size_t value_size;
  
  event->type = CCA_Comm_Event_Config;
  iptr = (uint32_t *) buf;
  iptr++;

  config = cca_config_new ();

  ptr = (char *) iptr;
  cca_config_set_key (config, ptr);
  
  key_size = strlen (ptr) + 1;
  if (buf_size > key_size + sizeof (uint32_t))
    {
      ptr += key_size;
      
      iptr = (uint32_t *) ptr;
      value_size = ntohl (*iptr);
      iptr++;
      
      cca_config_set_value (config, iptr, value_size);
    }
  
  event->event_data.config = config;
}

void
cca_buffer_from_comm_event_protocol_mismatch (char ** buf_ptr, size_t * buf_size_ptr, cca_protocol_t protocol)
{
  size_t buf_size = sizeof (uint32_t) + sizeof (cca_protocol_t);
  char * buf;
  uint32_t * iptr;
  
  buf = cca_malloc (buf_size);
  
  iptr = (uint32_t *) buf;
  *iptr = htonl (CCA_Comm_Event_Protocol_Mismatch);
  iptr++;
  
  *iptr = htonl (protocol);
  /* iptr++ */

  *buf_ptr = buf;
  *buf_size_ptr = buf_size;
}

void
cca_comm_event_from_buffer_protocol_mismatch (char * buf, size_t buf_size, cca_comm_event_t * event)
{
  uint32_t * iptr;
  
  iptr = (uint32_t *) buf;
  iptr++;

  cca_comm_event_set_protocol_mismatch (event, ntohl (*iptr));
}





/* for ping, pong and close */
void
cca_buffer_from_comm_event (char ** buf_ptr, size_t * buf_size_ptr, cca_comm_event_t * event)
{
  size_t buf_size = sizeof (uint32_t);
  char * buf;
  uint32_t * iptr;
  
  buf = cca_malloc (buf_size);
  
  iptr = (uint32_t *) buf;
  *iptr = htonl (event->type);
  iptr++;
  
  
  *buf_ptr = buf;
  *buf_size_ptr = buf_size;
}

void
cca_comm_event_from_buffer  (char * buf, size_t buf_size, cca_comm_event_t * event)
{
  event->type = ntohl (*((uint32_t *) buf));
}


/* EOF */
